"""Custom types for interfaces, both physical and logical."""

import re
from typing import Annotated

from annotated_types import Len
from orchestrator.domain.base import T
from pydantic import AfterValidator, BaseModel
from pydantic_forms.types import strEnum
from pydantic_forms.validators import validate_unique_list
from typing_extensions import Doc


class LAGMember(BaseModel):
    """A :term:`LAG` member interface that consists of a name and description."""

    interface_name: str
    interface_description: str | None = None

    def __hash__(self) -> int:
        """Calculate the hash based on the interface name and description, so that uniqueness can be determined."""
        return hash(self.interface_name)


def validate_interface_names_are_unique(interfaces: list[LAGMember]) -> list[LAGMember]:
    """Verify if interfaces are unique.

    Raises a :class:`ValueError` if the interfaces are not unique.

    :param interfaces: The list of interfaces.
    :type interfaces: list[:class:`utils.types.LAGMember`]

    :return: The list of interfaces
    :rtype: list[:class:`utils.types.LAGMember`]
    """
    interface_names = [member.interface_name for member in interfaces]
    if len(interface_names) != len(set(interface_names)):
        msg = "Interfaces must be unique."
        raise ValueError(msg)
    return interfaces


def validate_juniper_phy_interface_name(interface_name: str) -> str:
    """Validate that the provided interface name matches the expected pattern.

    The expected pattern for the interface name is one of 'ge', 'et', 'xe' followed by a dash '-',
    then a number between 0 and 19, a forward slash '/', another number between 0 and 99,
    another forward slash '/', and ends with a number between 0 and 99.
    For example: 'xe-1/0/0'. This only applies to Juniper-brand hardware.

    :param str interface_name: Interface name to validate.

    :return str: The interface name if match was successful, otherwise it will throw a ValueError exception.
    """
    pattern = re.compile(r"^(ge|et|xe)-1?[0-9]/[0-9]{1,2}/[0-9]{1,2}$")
    if not bool(pattern.match(interface_name)):
        error_msg = (
            f"Invalid interface name. The interface name should be of format: xe-1/0/0. " f"Got: {interface_name}"
        )
        raise ValueError(error_msg)
    return interface_name


def validate_juniper_ae_interface_name(interface_name: str) -> str:
    """Validate that the provided interface name matches the expected pattern for a :term:`LAG` interface.

    Interface names must match the pattern 'ae' followed by a one- or two-digit number.
    """
    juniper_lag_re = re.compile("^ae\\d{1,2}$")
    if not juniper_lag_re.match(interface_name):
        msg = "Invalid LAG name, please try again."
        raise ValueError(msg)
    return interface_name


JuniperPhyInterface = Annotated[str, AfterValidator(validate_juniper_phy_interface_name)]
JuniperAEInterface = Annotated[str, AfterValidator(validate_juniper_ae_interface_name)]
LAGMemberList = Annotated[
    list[T],
    AfterValidator(validate_unique_list),
    AfterValidator(validate_interface_names_are_unique),
    Len(min_length=0),
    Doc("A list of :term:`LAG` member interfaces."),
]


class JuniperLAGMember(LAGMember):
    """A Juniper-specific :term:`LAG` member interface."""

    interface_name: JuniperPhyInterface


class PhysicalPortCapacity(strEnum):
    """Physical port capacity enumerator.

    An enumerator that has the different possible capacities of ports that are available to use in subscriptions.
    """

    ONE_GIGABIT_PER_SECOND = "1G"
    TEN_GIGABIT_PER_SECOND = "10G"
    HUNDRED_GIGABIT_PER_SECOND = "100G"
    FOUR_HUNDRED_GIGABIT_PER_SECOND = "400G"
